@@ -1,13 +1,29 @@
+/*
+JonesCLI
+
+Author: Vlad Nedelcu
+Date: Jul 2021
+License: MIT
+
+Copyright 2021 Vlad Nedelcu
+*/
+
 pub mod utils;
 pub mod objects;
 pub mod display;
+pub mod markers;
+pub mod docstrings;
 
 use std::fs;
 use std::path::PathBuf;
 
-static CLASS_KEYWORD: &str = "class {template}";
-static TEMPLATE_KEYWORD: &str = "{template}";
-static PYTHON_EXTENSION: &str = "py";
+const CLASS_TEMPLATE_INHERITANCE: &str = "class {template}(";
+const CLASS_TEMPLATE: &str = "class {template}:";
+const TEMPLATE_KEYWORD: &str = "{template}";
+const PYTHON_EXTENSION: &str = "py";
+const DOCSTRING: &str = "\"\"\"";
+const NEWLINE: &str = "\n";
+
 
 type ClassMatch = (String, String);
 
@@ -21,25 +37,40 @@ type ClassMatch = (String, String);
 /// * `class_name`: The searched class name
 ///
 fn extract_python_class(code_lines: Vec<&str>, class_name: &str) -> objects::PythonClass {
-    let full_class_name = CLASS_KEYWORD.clone()
+    let class_name_inheritance = CLASS_TEMPLATE_INHERITANCE.clone()
+        .replace(TEMPLATE_KEYWORD, class_name);
+    let class_name_simple = CLASS_TEMPLATE.clone()
         .replace(TEMPLATE_KEYWORD, class_name);
 
     let mut start_cutting: bool = false;
     let mut class_code_block: Vec<String> = Vec::new();
+    let mut class_header: String = String::from("");
 
     for (counter, line) in code_lines.iter().enumerate() {
-        if line.contains(&full_class_name) {
+        if line.contains(&class_name_inheritance) || line.contains(&class_name_simple) {
             start_cutting = true;
+            class_header.push_str(line)
         }
         if start_cutting {
             class_code_block.push(line.to_string());
-            if line.is_empty() && code_lines[counter-1].is_empty() {
+
+            if line.len() <= 1 && code_lines[counter-1].len() <= 1 {
                 break
             }
         }
     }
 
-    objects::PythonClass::new(class_code_block, class_name.to_string())
+    let class_inheritance = match utils::extract_class_inheritance(&class_header) {
+        Some(inheritance_vec) => inheritance_vec,
+        None => Vec::new()
+    };
+
+    let docstring = match docstrings::extract_docstring(&class_code_block) {
+        Some(inheritance_vec) => inheritance_vec,
+        None => String::from("None")
+    };
+
+    objects::PythonClass::new(class_code_block, class_name.to_string(), class_inheritance, docstring)
 }
 
 /// Check if a file contains the searched class by reading the file.
@@ -51,12 +82,16 @@ fn extract_python_class(code_lines: Vec<&str>, class_name: &str) -> objects::Pyt
 /// # Errors
 /// It panics if the file is cannot be read properly
 fn check_file_contains_class(class_name: &str, file_path: &str) -> bool {
-    let full_class_name = CLASS_KEYWORD.clone()
+    let class_name_inheritance = CLASS_TEMPLATE_INHERITANCE.clone()
+        .replace(TEMPLATE_KEYWORD, class_name);
+    let class_name = CLASS_TEMPLATE.clone()
         .replace(TEMPLATE_KEYWORD, class_name);
 
     match fs::read_to_string(file_path) {
         Ok(file_content) => {
-            return file_content.contains(&full_class_name)
+            let first_check = file_content.contains(&class_name_inheritance);
+            let second_check = file_content.contains(&class_name);
+            return first_check || second_check
         },
         Err(_) => {
             return false
@@ -64,7 +99,8 @@ fn check_file_contains_class(class_name: &str, file_path: &str) -> bool {
     };
 }
 
-/// Searches recurssively through a project for a Python class
+/// Searches recurssively through a project for a Python class and extracts that
+/// class into an PythonClass struct.
 pub fn project_traversal(dir_path: &PathBuf, class_name: &String) -> Option<objects::PythonClass> {
     let current_dir = match fs::read_dir(dir_path) {
         Ok(dir) => dir,
@@ -92,7 +128,7 @@ pub fn project_traversal(dir_path: &PathBuf, class_name: &String) -> Option<obje
                     }
                 },
                 None => continue
-            };
+            }
             if check_file_contains_class(class_name, &file_path_name){
                 let file_content = match fs::read_to_string(file_path) {
                     Ok(content) => content,
@@ -170,7 +206,7 @@ mod tests {
 
     static PYTHON_CODE: &str = "
     class God:
-
+        \"\"\"DocString\"\"\"
         def __init__(self, name: int):
             self.name == name
 
@@ -193,6 +229,7 @@ mod tests {
 
         let test_codebase = vec![
             "class God:".to_string(),
+            "    \"\"\"DocString\"\"\"".to_string(),
             "".to_string(),
             "    def __init__(self, name: int):".to_string(),
             "        self.name = name".to_string(),
@@ -203,7 +240,7 @@ mod tests {
         ];
         let lines: Vec<&str> = PYTHON_CODE.split("\n").collect();
 
-        let expected_class = PythonClass::new(test_codebase, String::from("God"));
+        let expected_class = PythonClass::new(test_codebase, String::from("God"), Vec::new(), String::from("DocString"));
 
         assert_eq!(extract_python_class(lines, "God"), expected_class);
 
@@ -234,6 +271,7 @@ mod tests {
 
         let test_codebase = vec![
             "class God:".to_string(),
+            "    \"\"\"DocString\"\"\"".to_string(),
             "".to_string(),
             "    def __init__(self, name: int):".to_string(),
             "        self.name = name".to_string(),
@@ -243,7 +281,7 @@ mod tests {
             "".to_string()
         ];
 
-        let expected_class = PythonClass::new(test_codebase, String::from("God"));
+        let expected_class = PythonClass::new(test_codebase, String::from("God"), Vec::new(), String::from("DocString"));
         assert_eq!(expected_class, project_traversal(&pathbuf, &"God".to_string()).unwrap());
 
         fs::remove_dir_all("./testing").expect("Could not delete dir");
