@@ -1,5 +1,21 @@
+/*
+JonesCLI
+
+Author: Vlad Nedelcu
+Date: Jul 2021
+License: MIT
+
+Copyright 2021 Vlad Nedelcu
+*/
+
 use regex::Regex;
-use super::objects::{Parameter, Method};
+use super::{
+    objects::{
+        Parameter,
+        Method
+    },
+    markers::get_header_arguments
+};
 
 static FUNCTION_KEYWORD: &str = " def ";
 static DEFAULT_TYPE: &str = "None";
@@ -14,10 +30,9 @@ static CLASS_KEYWORD: &str = "class ";
 /// * `value`: value to split
 fn regex_split<'a>(r: &'a str, trim: bool, value: &'a String) -> Vec<&'a str> {
     let regex_separator = Regex::new(r).expect("Invalid regex function given");
-    if trim {
-        regex_separator.split(value.trim()).collect()
-    } else {
-        regex_separator.split(value).collect()
+    match trim {
+        true => regex_separator.split(value.trim()).collect(),
+        false => regex_separator.split(value).collect()
     }
 }
 
@@ -48,23 +63,21 @@ pub fn extract_method_name(method_header: &String) -> Result<String, &str> {
 pub fn extract_parameters(header: &String) -> Vec<Parameter> {
 
     // Split to get all the parameter
-    let params_between_parantheses: Vec<&str> = regex_split(r"(\(|\):|\)\s)", true, header);
-    let params: String = params_between_parantheses[1].to_string();
-    let params_values: Vec<&str> = regex_split(r",", true, &params);
+    let parameter_segment: String = match get_header_arguments(header) {
+        Some(params) => params,
+        None => return Vec::new()
+    };
+    let parameters_values = regex_split(r",\s", true, &parameter_segment);
     let mut parameters: Vec<Parameter> = Vec::new();
 
     /*
     Starting from the second value, iterate and extract
     all the parameters from the method
     */
-    for parameter in params_values.iter(){
+    for param in parameters_values.iter(){
 
-        let parameter_string = parameter.to_string();
-        if parameter.is_empty() {
-            continue
-        }
-
-        let param_values: Vec<&str> = regex_split(r":\s", false, &parameter_string);
+        let param_string = param.to_string();
+        let param_values: Vec<&str> = regex_split(r":\s", false, &param_string);
         match param_values.len() {
             1 => {
                 parameters.push(
@@ -82,7 +95,7 @@ pub fn extract_parameters(header: &String) -> Vec<Parameter> {
                     )
                 )
             },
-            _ => println!("Found method in code with no parameters.")
+            _ => println!("Found method in code with no parameters. {:?}", param_values)
         }
 
     }
@@ -100,7 +113,7 @@ pub fn extract_methods(class_code: Vec<String>) -> Vec<Method> {
     // Initialize temp method and start for retrieving method headers
     // that span on multiple lines
     let mut methods: Vec<Method> = Vec::new();
-    let mut temp_method = String::from("");
+    let mut temp_method = String::new();
     let mut start = false;
 
     for line in class_code.iter() {
@@ -108,7 +121,7 @@ pub fn extract_methods(class_code: Vec<String>) -> Vec<Method> {
             start = true;
         }
         if start {
-            temp_method.push_str(line.trim());
+            temp_method.push_str(format!(" {}", line.trim()).as_str());
             let last_char = match temp_method.chars()
                 .nth(temp_method.len() - 1) {
                     Some(chr) => chr,
@@ -117,7 +130,7 @@ pub fn extract_methods(class_code: Vec<String>) -> Vec<Method> {
 
             if last_char == ENDEF_KEYWORD {
                 methods.push(Method::new(&temp_method));
-                temp_method = String::from("");
+                temp_method = String::new();
                 start = false;
             }
         }
@@ -138,13 +151,8 @@ pub fn extract_methods(class_code: Vec<String>) -> Vec<Method> {
 pub fn extract_method_output(header: &String) -> Result<String, &str> {
     let header_split = regex_split(r"( -> )", true, header);
     match header_split.len() {
-        1 => {
-            return Err("Output type not found")
-        },
-        _ => {
-            let cleaned_output = header_split[1].trim().replace(":", "");
-            return Ok(cleaned_output)
-        }
+        1 => Err("Output type not found"),
+        _ => Ok(header_split[1].trim().replace(":", ""))
     }
 }
 
@@ -167,14 +175,41 @@ pub fn grep_class<'a>(lines: Vec<&str>, keyword: &String, file_name: &str) -> Op
             );
         }
     }
-    if found_match_classes.len() > 0 {
-        Some(found_match_classes)
-    } else {
-        None
+    match found_match_classes.len() {
+        0 => None,
+        _ => Some(found_match_classes)
     }
 }
 
 
+/// Extract class inheritance objects
+///
+/// > Note: Does not include extracting filters or constraints
+///
+/// # Arguments
+///
+/// * `line` - Is the class header previously extracted
+///
+/// # Output
+///
+/// A vector containing all the objects the class inherits
+pub fn extract_class_inheritance(line: &String) -> Option<Vec<String>> {
+
+    // Split the header for now to get objects that are inherited by the class
+    let class_header_split: Vec<&str> = regex_split(r"(\(|\):|,)", true, line);
+    if class_header_split.len() == 1 {
+        return None
+    }
+
+    // Iterate through the split results but ommit the first entry
+    let mut class_inheritance: Vec<String> = Vec::new();
+    for inheritance in class_header_split[1..class_header_split.len()-1].iter(){
+        let inherit_object = inheritance.trim().to_string();
+        class_inheritance.push(inherit_object);
+    }
+
+    Some(class_inheritance)
+}
 
 #[cfg(test)]
 mod tests {
@@ -341,4 +376,19 @@ mod tests {
 
         assert_eq!(grep_class(test_codebase, &keyword, filename), None);
     }
+
+    #[test]
+    fn test_extract_class_inheritance() {
+        let test_header = String::from("class Human(Being, Earthling):");
+        let expected = vec![String::from("Being"), String::from("Earthling")];
+
+        assert_eq!(extract_class_inheritance(&test_header), Some(expected));
+    }
+
+    #[test]
+    fn test_extract_no_inheritance(){
+        let test_header = String::from("class Human:");
+
+        assert_eq!(extract_class_inheritance(&test_header), None);
+    }
 }
\ No newline at end of file
